@@ -1,6 +1,6 @@
 """
 
-Helper functions for writing to terminals and files. 
+Helper functions for writing to terminals and files.
 
 """
 
@@ -18,21 +18,21 @@ if sys.platform == "win32":
 
 def _getdimensions():
     import termios,fcntl,struct
-    call = fcntl.ioctl(0,termios.TIOCGWINSZ,"\000"*8)
+    call = fcntl.ioctl(1,termios.TIOCGWINSZ,"\000"*8)
     height,width = struct.unpack( "hhhh", call ) [:2]
-    return height, width 
+    return height, width
 
 
 def get_terminal_width():
     try:
         height, width = _getdimensions()
-    except (SystemExit, KeyboardInterrupt):
+    except py.builtin._sysex:
         raise
     except:
         # FALLBACK
         width = int(os.environ.get('COLUMNS', 80))
     else:
-        # XXX the windows getdimensions may be bogus, let's sanify a bit 
+        # XXX the windows getdimensions may be bogus, let's sanify a bit
         if width < 40:
             width = 80
     return width
@@ -47,7 +47,7 @@ def ansi_print(text, esc, file=None, newline=True, flush=False):
     if esc and not isinstance(esc, tuple):
         esc = (esc,)
     if esc and sys.platform != "win32" and file.isatty():
-        text = (''.join(['\x1b[%sm' % cod for cod in esc])  +  
+        text = (''.join(['\x1b[%sm' % cod for cod in esc])  +
                 text +
                 '\x1b[0m')     # ANSI color code "reset"
     if newline:
@@ -78,9 +78,11 @@ def ansi_print(text, esc, file=None, newline=True, flush=False):
             handle = GetStdHandle(STD_ERROR_HANDLE)
         else:
             handle = GetStdHandle(STD_OUTPUT_HANDLE)
+        oldcolors = GetConsoleInfo(handle).wAttributes
+        attr |= (oldcolors & 0x0f0)
         SetConsoleTextAttribute(handle, attr)
         file.write(text)
-        SetConsoleTextAttribute(handle, FOREGROUND_WHITE)
+        SetConsoleTextAttribute(handle, oldcolors)
     else:
         file.write(text)
 
@@ -93,9 +95,9 @@ def should_do_markup(file):
            and not (sys.platform.startswith('java') and os._name == 'nt')
 
 class TerminalWriter(object):
-    _esctable = dict(black=30, red=31, green=32, yellow=33, 
+    _esctable = dict(black=30, red=31, green=32, yellow=33,
                      blue=34, purple=35, cyan=36, white=37,
-                     Black=40, Red=41, Green=42, Yellow=43, 
+                     Black=40, Red=41, Green=42, Yellow=43,
                      Blue=44, Purple=45, Cyan=46, White=47,
                      bold=1, light=2, blink=5, invert=7)
 
@@ -106,19 +108,19 @@ class TerminalWriter(object):
             if stringio:
                 self.stringio = file = py.io.TextIO()
             else:
-                file = py.std.sys.stdout 
+                file = py.std.sys.stdout
                 if hasattr(file, 'encoding'):
                     encoding = file.encoding
         elif hasattr(file, '__call__'):
             file = WriteFile(file, encoding=encoding)
-        self.encoding = encoding 
+        self.encoding = encoding
         self._file = file
         self.fullwidth = get_terminal_width()
         self.hasmarkup = should_do_markup(file)
 
     def _escaped(self, text, esc):
         if esc and self.hasmarkup:
-            text = (''.join(['\x1b[%sm' % cod for cod in esc])  +  
+            text = (''.join(['\x1b[%sm' % cod for cod in esc])  +
                 text +'\x1b[0m')
         return text
 
@@ -183,12 +185,12 @@ class TerminalWriter(object):
 class Win32ConsoleWriter(TerminalWriter):
     def write(self, s, **kw):
         if s:
-            s = self._getbytestring(s)
-            if self.hasmarkup:
-                handle = GetStdHandle(STD_OUTPUT_HANDLE)
-
+            oldcolors = None
             if self.hasmarkup and kw:
-                attr = 0
+                handle = GetStdHandle(STD_OUTPUT_HANDLE)
+                oldcolors = GetConsoleInfo(handle).wAttributes
+                default_bg = oldcolors & 0x00F0
+                attr = default_bg
                 if kw.pop('bold', False):
                     attr |= FOREGROUND_INTENSITY
 
@@ -199,29 +201,31 @@ class Win32ConsoleWriter(TerminalWriter):
                 elif kw.pop('green', False):
                     attr |= FOREGROUND_GREEN
                 else:
-                    attr |= FOREGROUND_WHITE
+                    attr |= FOREGROUND_BLACK # (oldcolors & 0x0007)
 
                 SetConsoleTextAttribute(handle, attr)
+            if not isinstance(self._file, WriteFile):
+                s = self._getbytestring(s)
             self._file.write(s)
             self._file.flush()
-            if self.hasmarkup:
-                SetConsoleTextAttribute(handle, FOREGROUND_WHITE)
+            if oldcolors:
+                SetConsoleTextAttribute(handle, oldcolors)
 
     def line(self, s="", **kw):
         self.write(s+"\n", **kw)
 
-class WriteFile(object): 
-    def __init__(self, writemethod, encoding=None): 
-        self.encoding = encoding 
-        self._writemethod = writemethod 
+class WriteFile(object):
+    def __init__(self, writemethod, encoding=None):
+        self.encoding = encoding
+        self._writemethod = writemethod
 
     def write(self, data):
         if self.encoding:
             data = data.encode(self.encoding)
         self._writemethod(data)
 
-    def flush(self): 
-        return 
+    def flush(self):
+        return
 
 
 if win32_and_ctypes:
@@ -232,46 +236,50 @@ if win32_and_ctypes:
     # ctypes access to the Windows console
     STD_OUTPUT_HANDLE = -11
     STD_ERROR_HANDLE  = -12
+    FOREGROUND_BLACK     = 0x0000 # black text
     FOREGROUND_BLUE      = 0x0001 # text color contains blue.
     FOREGROUND_GREEN     = 0x0002 # text color contains green.
     FOREGROUND_RED       = 0x0004 # text color contains red.
     FOREGROUND_WHITE     = 0x0007
     FOREGROUND_INTENSITY = 0x0008 # text color is intensified.
+    BACKGROUND_BLACK     = 0x0000 # background color black
     BACKGROUND_BLUE      = 0x0010 # background color contains blue.
     BACKGROUND_GREEN     = 0x0020 # background color contains green.
     BACKGROUND_RED       = 0x0040 # background color contains red.
     BACKGROUND_WHITE     = 0x0070
     BACKGROUND_INTENSITY = 0x0080 # background color is intensified.
 
+    SHORT = ctypes.c_short
+    class COORD(ctypes.Structure):
+        _fields_ = [('X', SHORT),
+                    ('Y', SHORT)]
+    class SMALL_RECT(ctypes.Structure):
+        _fields_ = [('Left', SHORT),
+                    ('Top', SHORT),
+                    ('Right', SHORT),
+                    ('Bottom', SHORT)]
+    class CONSOLE_SCREEN_BUFFER_INFO(ctypes.Structure):
+        _fields_ = [('dwSize', COORD),
+                    ('dwCursorPosition', COORD),
+                    ('wAttributes', wintypes.WORD),
+                    ('srWindow', SMALL_RECT),
+                    ('dwMaximumWindowSize', COORD)]
+
     def GetStdHandle(kind):
         return ctypes.windll.kernel32.GetStdHandle(kind)
 
-    def SetConsoleTextAttribute(handle, attr):
-        ctypes.windll.kernel32.SetConsoleTextAttribute(
-            handle, attr)
+    SetConsoleTextAttribute = \
+        ctypes.windll.kernel32.SetConsoleTextAttribute
+
+    def GetConsoleInfo(handle):
+        info = CONSOLE_SCREEN_BUFFER_INFO()
+        ctypes.windll.kernel32.GetConsoleScreenBufferInfo(\
+            handle, ctypes.byref(info))
+        return info
 
     def _getdimensions():
-
-        SHORT = ctypes.c_short
-        class COORD(ctypes.Structure):
-            _fields_ = [('X', SHORT),
-                        ('Y', SHORT)]
-        class SMALL_RECT(ctypes.Structure):
-            _fields_ = [('Left', SHORT),
-                        ('Top', SHORT),
-                        ('Right', SHORT),
-                        ('Bottom', SHORT)]
-        class CONSOLE_SCREEN_BUFFER_INFO(ctypes.Structure):
-            _fields_ = [('dwSize', COORD),
-                        ('dwCursorPosition', COORD),
-                        ('wAttributes', wintypes.WORD),
-                        ('srWindow', SMALL_RECT),
-                        ('dwMaximumWindowSize', COORD)]
-        STD_OUTPUT_HANDLE = -11
         handle = GetStdHandle(STD_OUTPUT_HANDLE)
-        info = CONSOLE_SCREEN_BUFFER_INFO()
-        ctypes.windll.kernel32.GetConsoleScreenBufferInfo(
-            handle, ctypes.byref(info))
+        info = GetConsoleInfo(handle)
         # Substract one from the width, otherwise the cursor wraps
         # and the ending \n causes an empty line to display.
         return info.dwSize.Y, info.dwSize.X - 1
